#ifndef MESH_HPP
#define MESH_HPP

#include <ItemValue.hpp>
#include <TinyVector.hpp>

#include <memory>

struct IMesh
{
  virtual const size_t meshDimension() const = 0;
  ~IMesh() = default;
};

template <typename ConnectivityType>
class Mesh final : public IMesh
{
public:
  using Connectivity = ConnectivityType;

  static constexpr size_t dimension = ConnectivityType::dimension;
  using Rd = TinyVector<dimension>;

private:
  const std::shared_ptr<Connectivity> m_connectivity;
  NodeValue<const Rd> m_xr;
  NodeValue<Rd> m_mutable_xr;

public:
  KOKKOS_INLINE_FUNCTION
  const size_t meshDimension() const
  {
    return dimension;
  }

  KOKKOS_INLINE_FUNCTION
  const Connectivity& connectivity() const
  {
    return *m_connectivity;
  }

  KOKKOS_INLINE_FUNCTION
  size_t numberOfNodes() const
  {
    return m_connectivity->numberOfNodes();
  }

  KOKKOS_INLINE_FUNCTION
  size_t numberOfFaces() const
  {
    return m_connectivity->numberOfFaces();
  }

  KOKKOS_INLINE_FUNCTION
  size_t numberOfCells() const
  {
    return m_connectivity->numberOfCells();
  }

  KOKKOS_INLINE_FUNCTION
  const NodeValue<const Rd>& xr() const
  {
    return m_xr;
  }

  [[deprecated("should rework this class: quite ugly")]]
  KOKKOS_INLINE_FUNCTION
  NodeValue<Rd> mutableXr() const
  {
    return m_mutable_xr;
  }

  KOKKOS_INLINE_FUNCTION
  Mesh(const std::shared_ptr<Connectivity>& connectivity,
       NodeValue<Rd>& xr)
      : m_connectivity{connectivity},
        m_xr{xr},
        m_mutable_xr{xr}
  {
    ;
  }

  Mesh() = delete;

  ~Mesh()
  {
    ;
  }
};

#endif // MESH_HPP
